package org.ow2.odis.proceed.admin;

import java.net.URL;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.odis.connection.context.Context;
import org.ow2.odis.engine.FWEvent;
import org.ow2.odis.routing.Destination;
import org.ow2.odis.routing.IRouting;
import org.ow2.odis.routing.RoutingObject;
import org.ow2.odis.routing.RoutingObjectFactory;
import org.ow2.odis.exception.OdisException;
import org.ow2.odis.exception.OdisFactoryException;
import org.ow2.odis.proceed.ProceedManager;

import org.ow2.odis.proceed.admin.commands.HelpCommand;
import java.io.FileNotFoundException;

/**
 * This Proceed provides a command interpretor that get command String, 
 * run the appropriate command and return a report.
 * 
 * @author Arnaud Constancin - OrangeLabs
 * @author Olivier Beyler - OrangeLabs
 */
public class AdminProceed implements IRouting {

    // private static Logger logger = Logger.getLogger(AdminProceed.class);
    private static final Logger LOGGER = Monolog.initialize().getLogger(AdminProceed.class.getName());
    private List commandsList = new ArrayList();
    AdminCommandFactory factory = new AdminCommandFactory();
    RoutingObjectFactory ROfactory;
    /*
     * @see com.francetelecom.odis.component.engine.IProceed#init(java.lang.String)
     */
    public boolean init(String value) {
        boolean result = true;
        ROfactory= new RoutingObjectFactory(this);
        //
        // default commands
        addCommand(new HelpCommand(commandsList));

        if (value == null || value.trim().length() == 0) {
            value = "file:AdminNode.conf";
        } else {
            value = "/" + value;
        }
        //
        // personalized commands
        URL adminNodeConfig = null;
        try {
            adminNodeConfig = getClass().getResource(value);
            if (adminNodeConfig == null) {
                throw new FileNotFoundException();
            }
            List commandClassNames = null;
            ConfigReader cr = new ConfigReader(adminNodeConfig);
            commandClassNames = cr.load();
            List commandInstances = getCommandList(commandClassNames);
            Iterator it = commandInstances.iterator();
            while (it.hasNext()) {
                IAdminCommand iac = (IAdminCommand) it.next();
                addCommand(iac);
            }
        } catch (Exception e) {
            LOGGER.log(BasicLevel.ERROR, e);
            result = false;
        }

        return result;
    }

    /**
     * @return
     */
    public List getCommandList(List commandClassNames) {
        List commandInstances = new ArrayList();

        if (null != commandClassNames) {

            IAdminCommand iac = null;
            Iterator it = commandClassNames.iterator();
            while (it.hasNext()) {
                String commandClassName = (String) it.next();
                try {
                    LOGGER.log(BasicLevel.DEBUG, "Get instance of " + commandClassName);
                    iac = (IAdminCommand) factory.getInstanceOf(commandClassName);
                    LOGGER.log(BasicLevel.DEBUG, "Add command : " + commandClassName);
                    commandInstances.add(iac);
                } catch (OdisFactoryException e) {
                    e.printStackTrace();
                }

            }
        }

        return commandInstances;
    }

    /**
     * @param command
     */
    protected void addCommand(IAdminCommand command) {
        commandsList.add(command);
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.francetelecom.odis.component.engine.IProceed#proceed(java.lang.Object)
     */
    public List proceed(Object obj) {      

        List result = new ArrayList();
        RoutingObject ro= ROfactory.newRoutingObject();
        String errorMessage = "";

        CommandReport report = new CommandReport();
        report.setStartTime(System.currentTimeMillis());

        if (null == obj) {
            errorMessage = "[AdminProceed] Admin Node does'n deal with null object !";
            LOGGER.log(BasicLevel.ERROR, errorMessage);
            return null;
        } else if (!(obj instanceof String)) {
            errorMessage = "[AdminProceed] Admin Node only deals with String objects !";
            LOGGER.log(BasicLevel.ERROR, errorMessage);
            report.addError(errorMessage);
        } else {
            String commandText = (String) obj;
            try {
                processCommand(commandText, report);
            } catch (OdisException e) {
                report.addError("OdisException" + e.getMessage());
                e.printStackTrace();
            }
        }

        report.setEndTime(System.currentTimeMillis());
        Context context =ProceedManager.getCurrentReceiveContext(this);
        System.out.println("RECEIVE FROM "+context.toString());
        ro.addDestination(new Destination("Admin",context));
        ro.setPayload(report.toString());        
        result.add(ro);

        if (BasicLevel.DEBUG == LOGGER.getCurrentIntLevel()) {
            StringBuffer debug= new StringBuffer("[AdminProceed] admin procced result data = ");
            debug.append(report.toString());
            LOGGER.log(BasicLevel.DEBUG, debug.toString());
        } // else : nothing to do

        return result;
    }

    /**
     * @param commandText
     * @param report
     */
    protected void processCommand(String commandText, CommandReport report) throws OdisException {
        String errorMessage = "";

        // String commandName = "";
        boolean isMatching = false;

        String[] commandParts = decomposeCommand(commandText, report);
        report.setCommandId(commandParts[0]);
        report.setCommandName(commandParts[1]);

        // find command object and execute
        Iterator cmdIter = commandsList.iterator();
        while (cmdIter.hasNext() && !isMatching) {
            IAdminCommand adminCommand = (IAdminCommand) cmdIter.next();
            isMatching = adminCommand.matchesWithCommand(commandParts[1]);

            if (isMatching) {
                try {
                    // commandName = adminCommand.getName();
                    String commandResponse = adminCommand.run(commandParts[2]);
                    report.setCommandResponse(commandResponse);
                    LOGGER.log(BasicLevel.INFO, "commandResponse = " + commandResponse);
                } catch (AdminCommandException e) {
                    e.printStackTrace();
                    errorMessage = "ERROR while running command " + adminCommand.getName() + " : " + e.getMessage();
                    LOGGER.log(BasicLevel.ERROR, errorMessage, e);
                    report.addError(errorMessage);
                }
            }
        }

        if (!isMatching) {

            errorMessage = "command not found !" + commandText;
            LOGGER.log(BasicLevel.ERROR, errorMessage);
            report.addError(errorMessage);
        }

    }

    /**
     * extract Id, Name and args from the text of the command.
     * 
     * if no id then commandParts[0] = "";
     * 
     * @param commandText
     * @return commandParts : commandParts[0] = command.Id, commandParts[1] = command.Name, commandParts[0] = command.Args
     */
    protected String[] decomposeCommand(String commandText, CommandReport report) {

        String[] commandParts = new String[3];
        String rest = commandText.trim();
        int index = 0;
        commandText = commandText.trim();

        // get command id
        commandParts[0] = "";
        if (rest.startsWith("[")) {
            index = commandText.indexOf("]");
            if (index > 0) {
                commandParts[0] = rest.substring(1, index);
                rest = rest.substring(index + 1).trim();
            } else {
                report.addError("Malformed commandId : ']' is missing");
            }
        }

        // get command name
        index = rest.indexOf(" ");
        if (index > 0) {
            commandParts[1] = rest.substring(0, index);
            rest = rest.substring(index + 1).trim();
        } else {
            // there is no arguments
            commandParts[1] = rest;
            rest = null;
        }

        // get command arguments
        commandParts[2] = rest;

        LOGGER.log(BasicLevel.DEBUG, "COMMAND = " + commandParts[0] + " / " + commandParts[1] + " / " + commandParts[2]);

        return commandParts;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * com.francetelecom.odis.component.engine.IProceed#notifyFWEvent(com.francetelecom.odis.component.engine.FWEvent)
     */
    public void notifyFWEvent(FWEvent arg0) {
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.francetelecom.odis.component.engine.IProceed#close()
     */
    public void close() {
    }


    public List getLsDestinations() {
     List result= new ArrayList();
     result.add("Admin");
     return result;   
    }
}
