package org.ow2.odis.proceed.console;

import java.awt.Component;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Properties;
import javax.swing.JFileChooser;
import javax.swing.SwingUtilities;
import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.odis.connection.context.Context;
import org.ow2.odis.engine.FWEvent;
import org.ow2.odis.engine.IProceed;
import org.ow2.odis.node.Node;
import org.ow2.odis.proceed.IDeclare;
import org.ow2.odis.proceed.ProceedManager;

/**
 * The Class Console.
 */
public class Console extends AbstractConsole implements IProceed, IDeclare {
    // ----------------------------------------------------------------------------------------
    // attributes
    // ----------------------------------------------------------------------------------------
    /** The Constant LOGGER. */
    private static final Logger LOGGER = Monolog.initialize().getLogger(Console.class.getName());

    /** The Constant serialVersionUID. */
    private static final long serialVersionUID = 1L;

    /** The Constant EOL. */
    static final String EOL = System.getProperty("line.separator");

    /** The time offset. */
    private final transient long timeOffset = System.currentTimeMillis();

    /** The is pause. */
    private transient boolean isPause = false;

    /** The component. */
    protected transient Component component = null;

    /** The format class. */
    private String formatClass = "";

    /** The consumable classes. */
    private final transient String[] consummableClasses = { Object.class.getName() };

    /** The generated classes. */
    private final transient String[] generatedClasses = {};

    /** This Console or its ConsoleFacade. (For testing only.) */
    private IProceed thisConsole;

    /** Constructor. */
    public Console() {
        super();
        thisConsole = this;
    }

    /** 
     * Constructor for use with a facade.
     * @param facade 
     */
    public Console(IProceed facade) {
        this();
        thisConsole = facade;
    }

    /**
     * The main method.
     * @param args the args
     */
    public static void main(final String[] args) {
        Console console;
        console = new Console();
        console.setVisible(true);
        if (args.length == 1) {
            console.init(args[0]);
        }
    }

    /**
     * ask for a file name and save data to a file.
     * @see org.ow2.odis.proceed.console.AbstractConsole#saveModel()
     */
    protected void saveModel() {
        final JFileChooser fileChooser = new JFileChooser();
        if (fileChooser.showSaveDialog(this) == JFileChooser.APPROVE_OPTION) {
            getJButtonSave().setEnabled(false);
            final File file = fileChooser.getSelectedFile();
            final String path = file.getAbsolutePath();
            if (LOGGER.isLoggable(BasicLevel.DEBUG)) {
                LOGGER.log(BasicLevel.DEBUG, "chose file name\"" + path + "\"");
            }
            try {
                file.createNewFile();
                final RandomAccessFile fichier = new RandomAccessFile(path, "rw");
                final int col = getTableCommandModel().getColumnCount();
                final int raw = getTableCommandModel().getRowCount();
                for (int y = 0; y < raw; y++) {
                    for (int x = 0; x < col; x++) {
                        Object ob = getTableCommandModel().getValueAt(y, x);
                        fichier.writeBytes(ob.toString() + "|");
                    }
                    fichier.writeChars(EOL);
                }
                fichier.close();
            } catch (FileNotFoundException e) {
                LOGGER.log(BasicLevel.ERROR, "Unable to find file \"" + path + "\"", e);
            } catch (IOException e) {
                LOGGER.log(BasicLevel.ERROR, "IOException while accessing to file \"" + path + "\"", e);
            }
        }
    }

    /**
     * Proceed.
     * @param obj the obj
     * @return always null
     * @see com.francetelecom.odis.component.engine.IProceed#proceed(java.lang.Object)
     */
    public List proceed(Object obj) {
        if (isPause != true) {
            Object[] raw = new Object[4];
            long time = System.currentTimeMillis();
            SimpleDateFormat df = (SimpleDateFormat) SimpleDateFormat.getInstance();
            df.applyPattern("yyyy.MM.dd 'at' HH:mm:ss");
            Context context = ProceedManager.getCurrentReceiveContext(thisConsole);
            raw[0] = context;
            raw[1] = df.format(new Date(time));
            raw[2] = "" + (time - timeOffset);
            raw[3] = obj;
            SwingUtilities.invokeLater(new Update(raw));
        }
        return null;
    }

    /**
     * The Class Update.
     */
    class Update implements Runnable {
        /** The raw. */
        private Object[] raw;

        /**
         * The Constructor.
         * @param raw the raw
         */
        Update(Object[] raw) {
            super();
            this.raw = raw;
        }

        /**
         * Run.
         * @see java.lang.Runnable#run()
         */
        public void run() {
            getTableCommandModel().addValue(raw);
            getJButtonPurge().setEnabled(true);
            getJButtonSave().setEnabled(true);
            changeSelection();
        }
    }

    /**
     * Init.
     * @param value the value
     * @return true if succeed false in other case
     * @see com.francetelecom.odis.component.engine.IProceed#init(java.lang.String)
     */
    public boolean init(String value) {
        Node node = ProceedManager.getInstance().getNode(thisConsole);
        String description="console";
        if (node!=null)
            setTitle(node.getDescription());
        final Properties properties = new Properties();
        if (!("".equals(value))) {
            try {
                final URL url = getClass().getResource("/" + value);
                if (url == null) {
                    throw new FileNotFoundException();
                }
                properties.load(new FileInputStream(url.getPath()));
                description = properties.getProperty("description", "." + File.separatorChar + "run" + File.separatorChar + "in"
                        + File.separatorChar);
                formatClass = properties.getProperty("formatClass", "");
                final String sizeMaxRaw = properties.getProperty("sizeMaxRaw", "-1");
                try {
                    final int size = Integer.parseInt(sizeMaxRaw);
                    getTableCommandModel().setSizeMaxRaw(size);
                } catch (NumberFormatException e) {
                    LOGGER.log(BasicLevel.ERROR, "unable to set the sizeMaxRaw property with the value :" + sizeMaxRaw);
                }
            } catch (FileNotFoundException e) {
                LOGGER.log(BasicLevel.ERROR, "Unable to find " + value, e);
            } catch (IOException e) {
                LOGGER.log(BasicLevel.ERROR, "IOException on file " + value, e);
            }
        }
        getJTextField().setText(description);
        if ("".equals(formatClass)) {
            addTab(ConsoleTabDetail.class.getName());
        } else {
            addTab(formatClass);
        }
        setVisible(true);
        return true;
    }

    /**
     * Reset table.
     * @see org.ow2.odis.proceed.console.AbstractConsole#resetTable()
     */
    protected void resetTable() {
        getJButtonPurge().setEnabled(false);
        getJButtonSave().setEnabled(false);
        getTableCommandModel().clearValue();
    }

    /**
     * Stop acquiring.
     * @see org.ow2.odis.proceed.console.AbstractConsole#stopAcquiring()
     */
    protected void stopAcquiring() {
        isPause = !isPause;
        if (isPause) {
            getJButtonStop().setText("RESTART");
            getJButtonStop().setToolTipText("Restart acquiring");
        } else {
            getJButtonStop().setText("STOP");
            getJButtonStop().setToolTipText("Stop acquiring");
        }
    }

    /**
     * Change selection.
     * @see org.ow2.odis.proceed.console.AbstractConsole#changeSelection()
     */
    protected void changeSelection() {
        final int selectedRow = getJTable().getSelectedRow();
        if (selectedRow >= 0) {
            final Object ob = getTableCommandModel().getValueAt(selectedRow, 3);
            getJDescription().setObject(ob);
        }
    };

    /**
     * Adds the tab.
     * @param formatTab the format tab
     */
    protected void addTab(String formatTab) {
        try {
            final Class classe = Class.forName(formatTab);
            component = (Component) classe.newInstance();
            if (!(component instanceof ITabDetail)) {
                LOGGER.log(BasicLevel.ERROR, "class " + formatTab + " doesn't implement ITabDetail interface -> can't used it");
            }
            addDetailOnTab();
        } catch (ClassNotFoundException e) {
            LOGGER.log(BasicLevel.ERROR, "unable to found " + formatTab + " class", e);
        } catch (InstantiationException e) {
            LOGGER.log(BasicLevel.ERROR, "unable to instanciate " + formatTab + " class", e);
        } catch (IllegalAccessException e) {
            LOGGER.log(BasicLevel.ERROR, "illegal access on " + formatTab + " class", e);
        } catch (ClassCastException e) {
            LOGGER.log(BasicLevel.ERROR, "the Console Tab detail class choise is invalid " + formatTab + " class", e);
        }
    }

    /**
     * Adds the detail on tab.
     */
    protected void addDetailOnTab() {
        if (component != null) {
            getJSplitPane().setRightComponent(component);
            component.setVisible(true);
            getJSplitPane().updateUI();
        }
    }

    /**
     * This method initializes jDescription.
     * @return com.francetelecom.odis.component.engine.console.ConsoleTabDetail
     */
    protected ITabDetail getJDescription() {
        if (component == null) {
            component = new ConsoleTabDetail();
            component.setVisible(true);
            getJSplitPane().setRightComponent(component);
        }
        return (ITabDetail) component;
    }

    /**
     * Gets the consumable classes.
     * @return the consumable classes
     * @see com.francetelecom.odis.component.proceed.IDeclare#getConsumableClasses()
     */
    public String[] getConsumableClasses() {
        return consummableClasses;
    }

    /**
     * Gets the generate classes.
     * @return the generate classes
     * @see com.francetelecom.odis.component.proceed.IDeclare#getGenerateClasses()
     */
    public String[] getGenerateClasses() {
        return generatedClasses;
    }

    /**
     * Close.
     * @see com.francetelecom.odis.component.engine.IProceed#close()
     */
    public void close() {
        setVisible(false);
    }

    /**
     * Notify FW event.
     * @param event the event
     * @see com.francetelecom.odis.component.engine.IProceed#notifyFWEvent(com.francetelecom.odis.component.engine.FWEvent)
     */
    public void notifyFWEvent(FWEvent event) {
    }
}
