package org.ow2.odis.proceed.console;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.odis.engine.FWEvent;
import org.ow2.odis.engine.IProceed;

/**
 * A facade for the Console proceed.<br>
 * ConsoleFacade will try to use an Console and forward all calls to it. If a GUI cannot be used, ConsoleFacade will log to Java console.<br>
 * This is to prevent test cases to fail during server builds.
 * @author ojoh6423
 */
public class ConsoleFacade implements IProceed {

    /**
     * Denotes that a GUI is available.
     */
    private boolean useGUI = true;
    /**
     * The Console behind the facade (when a GUI is available).
     */
    private Console console = null;

    /** The Constant LOGGER. */
    private static final Logger LOGGER = Monolog.initialize().getLogger(ConsoleFacade.class.getName());

    /**
     * Constructor.<br>
     * Try to create an Console. If this is not possible, we will run without GUI (doing nothing).
     */
    public ConsoleFacade() {
        if (useGUI) {
            try {
                console = new Console(this);
            } catch (java.awt.AWTError e) {
                LOGGER.log(BasicLevel.WARN,"No GUI available. Can't create Console. Facade run only.");
                useGUI = false;
            } catch (java.awt.HeadlessException e) {
                LOGGER.log(BasicLevel.WARN,"No GUI available. Can't create Console. Facade run only.");
                useGUI = false;
            } catch (Error e) {
                LOGGER.log(BasicLevel.WARN," Can't create graphic. Facade run only.");
                useGUI = false;
            }
        }
    }

    /**
     * If a GUI is available, let the Console close. Otherwise do nothing.
     */
    public void close() {
        if (useGUI) {
            console.close();
        }
    }

    /**
     * If a GUI is available, let the Console init. Otherwise do nothing.
     * @param value location of a resource used to init the Console
     * @return the Console return value if present, or true
     */
    public boolean init(String value) {
        boolean b = true;
        if (useGUI) {
            try {
                b = console.init(value);
            } catch (java.awt.AWTError e) {
                LOGGER.log(BasicLevel.WARN,"No GUI available. Can't create Console. Facade run only.");
                useGUI = false;
            } catch (java.awt.HeadlessException e) {
                LOGGER.log(BasicLevel.WARN,"No GUI available. Can't create Console. Facade run only.");
                useGUI = false;
            } catch (Error e) {
                LOGGER.log(BasicLevel.WARN," Can't create graphic. Facade run only.");
                useGUI = false;
            }
        }
        return b;
    }

    /**
     * If a GUI is available, forward to Console. Otherwise do nothing.
     * @param event the event
     */
    public void notifyFWEvent(FWEvent event) {
        if (useGUI) {
            console.notifyFWEvent(event);
        }
    }

    /**
     * If a GUI is available, have the Console proceed the obj. Otherwise do nothing.
     * @param obj the object to proceed
     * @return the objects produced by the Console if present, or null
     */
    public List proceed(Object obj) {
        if (useGUI) {
            return console.proceed(obj);
        } else {
            long time = System.currentTimeMillis();
            SimpleDateFormat df = (SimpleDateFormat) SimpleDateFormat.getInstance();
            df.applyPattern("yyyy.MM.dd 'at' HH:mm:ss");
            System.out.println(df.format(new Date(time)) + " : " + obj.toString());
        }
        return null;
    }
}
