package org.ow2.odis.proceed.injector;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import javax.swing.JFileChooser;

import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.odis.connection.context.Context;
import org.ow2.odis.engine.FWEvent;
import org.ow2.odis.engine.IProceed;
import org.ow2.odis.proceed.ProceedManager;
import org.ow2.odis.routing.Destination;
import org.ow2.odis.routing.IRouting;
import org.ow2.odis.routing.RoutingObject;
import org.ow2.odis.routing.RoutingObjectFactory;


/**
 * The Class Injector.
 */
public class ContextualInjector extends AbstractContextualInjector implements IRouting {
    /** The Constant serialVersionUID. */
    private static final long serialVersionUID = 1L;

    /** The Constant LOGGER. */
    private static final Logger LOGGER = Monolog.initialize().getLogger(Injector.class.getName());

    /** The arbitrary destination name. */
    private static final String destinationName = "goodway";

    /** True if the automated event generation thread is active. */
    private boolean running = false;

    /** Communication zone between threads */
    private String[] lines;

    /** The present instance */
    private IProceed thisInjector;

    /** The associated RoutingObjectFactory in case contexts are used. */
    private RoutingObjectFactory rof = null;

    /** Instantiates a new injector. */
    public ContextualInjector() {
        super();
        thisInjector = this;
        rof = new RoutingObjectFactory(this);
    }

    /**
     * Instantiates a new injector through an facade.<br>
     * Used for testing purposes only.
     */
    public ContextualInjector(IProceed facade) {
        this();
        thisInjector = facade;
    }

    /** A list for event storage before sending. */
    private List lsEvent = new ArrayList();

    /**
     * Close. (Hides the GUI.)
     * @see com.francetelecom.odis.component.engine.IProceed#close()
     */
    public void close() {
        setVisible(false);
    }

    /**
     * Init. (Shows the GUI and gets the properties.)
     * @param value the value
     * @return true, if init
     * @see com.francetelecom.odis.component.engine.IProceed#init(java.lang.String)
     */
    public boolean init(String value) {
        setVisible(true);
        String description = ProceedManager.getInstance().getNode(thisInjector).getName();
         

        final Properties properties = new Properties();
        if (!("".equals(value))) {
            try {
                final URL url = getClass().getResource("/" + value);
                if (url == null) {
                    throw new FileNotFoundException();
                }
                properties.load(new FileInputStream(url.getPath()));
                description = properties.getProperty("description", "." + File.separatorChar + "run" + File.separatorChar + "in"
                        + File.separatorChar);
            } catch (FileNotFoundException e) {
                LOGGER.log(BasicLevel.ERROR, "Unable to find " + value, e);
            } catch (IOException e) {
                LOGGER.log(BasicLevel.ERROR, "IOException on file " + value, e);
            }
        }
        this.setTitle(description);
        return true;
    }

    /**
     * Notify FW event.
     * @param event the event
     * @see com.francetelecom.odis.component.engine.IProceed#notifyFWEvent(com.francetelecom.odis.component.engine.FWEvent)
     */
    public void notifyFWEvent(FWEvent event) {
    }

    /**
     * Proceed. (Send all pending events one by one.)
     * @param obj the obj
     * @return the list
     * @see com.francetelecom.odis.component.engine.IProceed#proceed(java.lang.Object)
     */
    public List proceed(Object obj) {
        List result = new ArrayList();
        if (obj == null) {
            while (!lsEvent.isEmpty()) {
                if (useContext()) {
                    RoutingObject ro = rof.newRoutingObject();
                    ro.addDestination(new Destination(destinationName, new Context(getIdentity(), getSubidentity(), getDetail())));
                    ro.setPayload(lsEvent.remove(0).toString());
                    result.add(ro);
                } else {
                    result.add(lsEvent.remove(0).toString());
                }
                eventSent();
            }
        } else {
            System.out.println("INJECTOR " + getName() + " RECEIVED A MESSAGE : ");
            System.out.println(obj.toString());
            System.out.println("=============================");
        }
        return result;
    }

    /**
     * Action when eventBtn gets clicked.<br>
     * Ignored if automated event generation running. Otherwise generates events according to user choices.
     * @see org.ow2.odis.proceed.injector.AbstractInjector#onEventBtn()
     */
    void onEventBtn() {
        if (running) {
            System.out.println("Automated event generation is currently active");
        } else {
            if (!isFull()) {
                if (isByLine()) {
                    lines = getEventText().split("\n");
                } else {
                    lines = getEventText().split(getSeparator());
                }
                if (isCycle()) {
                    running = true;
                    Thread t = new Thread() {
                        public void run() {
                            for (int i = 0; i < lines.length && isCycle(); i++) {
                                lsEvent.add(lines[i]);
                                ProceedManager.getInstance().activeProceed(thisInjector);
                                try {
                                    if (isCycle()) {
                                        Thread.sleep(getPeriod());
                                    }
                                } catch (InterruptedException e) {
                                    // Just wake up
                                }
                            }
                            running = false;
                        }
                    };
                    t.start();
                } else {
                    for (int i = 0; i < lines.length; i++) {
                        lsEvent.add(lines[i]);
                    }
                    ProceedManager.getInstance().activeProceed(thisInjector);
                }
            } else {
                running = true;
                Thread t = new Thread() {
                    public void run() {
                        do {
                            lsEvent.add(getEventText());
                            ProceedManager.getInstance().activeProceed(thisInjector);
                            try {
                                if (isCycle()) {
                                    Thread.sleep(getPeriod());
                                }
                            } catch (InterruptedException e) {
                                // Just wake up
                            }
                        } while (isCycle());
                        running = false;
                    }
                };
                t.start();
            }
        }
    }

    /**
     * Action when loadBtn is clicked.<br>
     * Displays a FileChooser and loads the chosen file contents into the text area.
     */
    void onLoadBtn() {
        final JFileChooser fileChooser = new JFileChooser();
        if (fileChooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
            final File file = fileChooser.getSelectedFile();
            final String path = file.getAbsolutePath();
            if (LOGGER.isLoggable(BasicLevel.DEBUG)) {
                LOGGER.log(BasicLevel.DEBUG, "chose file name\"" + path + "\"");
            }
            BufferedReader br = null;
            StringBuffer buffer = new StringBuffer();
            try {
                // flux de lecture en UTF-8
                br = new BufferedReader(new InputStreamReader(new FileInputStream(file)));
                // flux d'ecriture en ISO (valeur par defaut)
                String ligne;
                while ((ligne = br.readLine()) != null) {
                    buffer.append(ligne);
                    buffer.append("\n\r");
                }
            } catch (Exception e) {
                e.printStackTrace();
            } finally {
                try {
                    br.close();
                } catch (Throwable e) {
                }
            }
            setEventText(buffer.toString());
        }
    }

    /**
     * Action when resetBtn is clicked.<br>
     * Clears the text area.
     */
    void onResetBtn() {
        setEventText("");
    }

    /**
     * Provides the destinations this IProceed can address.
     * @return a list holding only the destination "goodway".
     */
    public List getLsDestinations() {
        List ls = new ArrayList(1);
        ls.add(destinationName);
        return ls;
    }
}
