package org.ow2.odis.proceed.injector;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.odis.engine.FWEvent;
import org.ow2.odis.engine.IProceed;

public class InjectorFacade implements IProceed {

    /**
     * Denotes that a GUI is available.
     */
    private boolean useGUI = true;
    /**
     * The Injector behind the facade (when a GUI is available).
     */
    private Injector injector = null;

    /** The Constant LOGGER. */
    private static final Logger LOGGER = Monolog.initialize().getLogger(InjectorFacade.class.getName());


    /**
     * Constructor.<br>
     * Try to create an Injector. If this is not possible, we will run without GUI (doing nothing).
     */
    public InjectorFacade() {
        if (useGUI) {
            try {
                injector = new Injector(this);
            } catch (java.awt.AWTError e) {
                LOGGER.log(BasicLevel.WARN,"No GUI available. Can't create Injector. Facade run only.");
                useGUI = false;
            } catch (java.awt.HeadlessException e) {
                LOGGER.log(BasicLevel.WARN,"No GUI available. Can't create Injector. Facade run only.");
                useGUI = false;
            } catch (Error e) {
                LOGGER.log(BasicLevel.WARN,"No GUI available. Can't create Injector. Facade run only.");
                useGUI = false;
            }

        }
    }

    /**
     * If a GUI is available, let the Injector close. Otherwise do nothing.
     */
    public void close() {
        if (useGUI) {
            injector.close();
        }
    }

    /**
     * If a GUI is available, let the Injector init. Otherwise do nothing.
     * @param value location of a resource used to init the Injector
     * @return the Injector return value if present, or true
     */
    public boolean init(String value) {
        boolean b = true;
        if (useGUI) {
            try {
                b = injector.init(value);
            } catch (java.awt.AWTError e) {
                LOGGER.log(BasicLevel.WARN,"No GUI available. Can't create Injector. Facade run only.");
                useGUI = false;
            } catch (java.awt.HeadlessException e) {
                LOGGER.log(BasicLevel.WARN,"No GUI available. Can't create Injector. Facade run only.");
                useGUI = false;
            } catch (Error e) {
                LOGGER.log(BasicLevel.WARN,"No GUI available. Can't create Injector. Facade run only.");
                useGUI = false;
            }
        }
        return b;
    }

    /**
     * If a GUI is available, forward to Injector. Otherwise do nothing.
     * @param event the event
     */
    public void notifyFWEvent(FWEvent event) {
        if (useGUI) {
            injector.notifyFWEvent(event);
        }
    }

    /**
     * If a GUI is available, have the Injector proceed the obj. Otherwise do nothing.
     * @param obj the object to proceed
     * @return the objects produced by the Injector if present, or null
     */
    public List proceed(Object obj) {
        if (useGUI) {
            return injector.proceed(obj);
        } else {
            long time = System.currentTimeMillis();
            SimpleDateFormat df = (SimpleDateFormat) SimpleDateFormat.getInstance();
            df.applyPattern("yyyy.MM.dd 'at' HH:mm:ss");
            System.out.println(df.format(new Date(time)) + " : " + obj.toString());
        }
        return null;
    }
}
